@@ -254,7 +254,13 @@ typedef struct notify_list_s *notify_list_t;
  * Buffer management layer.
  */
 
-#define BUFFER_BLOCK_SIZE  BUFSIZ
+/* BUFSIZ on Windows is 512 but on current Linux it is 8k.  We better
+ * use the 8k for Windows as well.  */
+#ifdef HAVE_W32_SYSTEM
+# define BUFFER_BLOCK_SIZE  8192
+#else
+# define BUFFER_BLOCK_SIZE  BUFSIZ
+#endif
 #define BUFFER_UNREAD_SIZE 16
 
 
@@ -473,8 +479,8 @@ const char *_gpgrt_fname_get (gpgrt_stream_t stream);
 
 
 #if HAVE_W32_SYSTEM
-/* Prototypes for w32-estream.c.  */
-struct cookie_io_functions_s _gpgrt_functions_w32_pollable;
+/* Prototypes for w32-estream.c. */
+extern struct cookie_io_functions_s _gpgrt_functions_w32_pollable;
 int _gpgrt_w32_pollable_create (void *_GPGRT__RESTRICT *_GPGRT__RESTRICT cookie,
                                 unsigned int modeflags,
                                 struct cookie_io_functions_s next_functions,
@@ -498,9 +504,9 @@ struct _gpgrt_b64state
   unsigned int crc;
   gpg_err_code_t lasterr;
   unsigned int flags;
-  int stop_seen:1;
-  int invalid_encoding:1;
-  int using_decoder:1;
+  unsigned int stop_seen:1;
+  unsigned int invalid_encoding:1;
+  unsigned int using_decoder:1;
 };
 
 gpgrt_b64state_t _gpgrt_b64enc_start (estream_t stream, const char *title);
