@@ -46,7 +46,6 @@
 #endif /*!HAVE_W32_SYSTEM*/
 #include <unistd.h>
 #include <fcntl.h>
-#include <assert.h>
 /* #include <execinfo.h> */
 
 #define _GPGRT_NEED_AFLOCAL 1
@@ -256,9 +255,10 @@ fun_writer (void *cookie_arg, const void *buffer, size_t size)
 #ifndef HAVE_W32_SYSTEM
           memset (&srvr_addr, 0, sizeof srvr_addr);
           srvr_addr_un.sun_family = af;
-          if (!*name && (name = socket_dir_cb ()) && *name)
+          if (!*name)
             {
-              if (strlen (name) + 7 < sizeof (srvr_addr_un.sun_path)-1)
+              if ((name = socket_dir_cb ()) && *name
+                  && strlen (name) + 7 < sizeof (srvr_addr_un.sun_path)-1)
                 {
                   strncpy (srvr_addr_un.sun_path,
                            name, sizeof (srvr_addr_un.sun_path)-1);
@@ -271,7 +271,7 @@ fun_writer (void *cookie_arg, const void *buffer, size_t size)
             }
           else
             {
-              if (*name && strlen (name) < sizeof (srvr_addr_un.sun_path)-1)
+              if (strlen (name) < sizeof (srvr_addr_un.sun_path)-1)
                 {
                   strncpy (srvr_addr_un.sun_path,
                            name, sizeof (srvr_addr_un.sun_path)-1);
@@ -690,13 +690,17 @@ _gpgrt_log_get_stream ()
     {
       /* Make sure a log stream has been set.  */
       _gpgrt_log_set_sink (NULL, NULL, -1);
-      assert (logstream);
+      if (!logstream)
+        {
+          fputs ("gpgrt fatal: failed to init log stream\n", stderr);
+          _gpgrt_abort ();
+        }
     }
   return logstream;
 }
 
 
-/* A fiter used with the fprintf_sf function to sanitize the args for
+/* A filter used with the fprintf_sf function to sanitize the args for
  * "%s" format specifiers.  */
 static char *
 fmt_string_filter (const char *string, int no, void *opaque)
@@ -902,7 +906,11 @@ _gpgrt_logv_internal (int level, int ignore_arg_ptr, const char *extrastring,
       /* Make sure a log stream has been set.  */
       _gpgrt_log_set_sink (NULL, NULL, -1);
 #endif
-      assert (logstream);
+      if (!logstream)
+        {
+          fputs ("gpgrt fatal: failed to init log stream\n", stderr);
+          _gpgrt_abort ();
+        }
     }
 
   _gpgrt_flockfile (logstream);
@@ -1038,7 +1046,7 @@ _gpgrt_logv_internal (int level, int ignore_arg_ptr, const char *extrastring,
       /*     for (btidx=0; btidx < btlen; btidx++) */
       /*       log_debug ("[%d] %s\n", btidx, btstr[btidx]); */
       /* } */
-      abort ();
+      _gpgrt_abort ();
     }
   else
     _gpgrt_funlockfile (logstream);
@@ -1136,7 +1144,7 @@ _gpgrt_log_fatal (const char *fmt, ...)
   va_start (arg_ptr, fmt);
   _gpgrt_logv_internal (GPGRT_LOGLVL_FATAL, 0, NULL, NULL, fmt, arg_ptr);
   va_end (arg_ptr);
-  abort (); /* Never called; just to make the compiler happy.  */
+  _gpgrt_abort (); /* Never called; just to make the compiler happy.  */
 }
 
 
@@ -1148,7 +1156,7 @@ _gpgrt_log_bug (const char *fmt, ...)
   va_start (arg_ptr, fmt);
   _gpgrt_logv_internal (GPGRT_LOGLVL_BUG, 0, NULL, NULL, fmt, arg_ptr);
   va_end (arg_ptr);
-  abort (); /* Never called; just to make the compiler happy.  */
+  _gpgrt_abort (); /* Never called; just to make the compiler happy.  */
 }
 
 
@@ -1331,5 +1339,5 @@ _gpgrt__log_assert (const char *expr, const char *file,
   _gpgrt_log (GPGRT_LOGLVL_BUG, "Assertion \"%s\" failed (%s:%d)\n",
            expr, file, line);
 #endif /*!GPGRT_HAVE_MACRO_FUNCTION*/
-  abort (); /* Never called; just to make the compiler happy.  */
+  _gpgrt_abort (); /* Never called; just to make the compiler happy.  */
 }
